"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _measurer = _interopRequireDefault(require("./measurer"));

var _getTreeSnapshot = _interopRequireDefault(require("./getTreeSnapshot"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function roundFloat(val) {
  var base = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 2;
  var n = Math.pow(10, base);
  return Math.floor(val * n) / n;
}

function getLastMeasurements() {
  if (process.env.NODE_ENV === 'production') {
    return [];
  }

  return _measurer.default.getData();
}

function start() {
  if (process.env.NODE_ENV === 'production') {
    return;
  }

  _measurer.default.begin();
}

function stop() {
  if (process.env.NODE_ENV === 'production') {
    return;
  }

  _measurer.default.end();
}

function getOperations() {
  var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getLastMeasurements();

  if (process.env.NODE_ENV === 'production') {
    return [];
  }

  var roots = _measurer.default.getRoots();

  var tree = _measurer.default.getTree();

  var treeSnapshot = (0, _getTreeSnapshot.default)(roots, tree);
  var stats = [];
  data.forEach(function (flush, flushIndex) {
    var operations = flush.operations;
    operations.forEach(function (operation) {
      var instanceID = operation.instanceID,
          type = operation.type,
          payload = operation.payload;

      var _ref = treeSnapshot[instanceID] || {},
          _ref$displayName = _ref.displayName,
          displayName = _ref$displayName === void 0 ? '' : _ref$displayName,
          _ref$parentID = _ref.parentID,
          parentID = _ref$parentID === void 0 ? 0 : _ref$parentID;

      var owner = treeSnapshot[parentID];
      var key = (owner ? owner.displayName + ' > ' : '') + displayName;
      stats.push({
        flushIndex: flushIndex,
        instanceID: instanceID,
        key: key,
        type: type,
        parentID: parentID,
        payload: payload
      });
    });
  });
  return stats;
}

function getInclusive() {
  var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getLastMeasurements();

  if (process.env.NODE_ENV === 'production') {
    return;
  }

  var roots = _measurer.default.getRoots();

  var tree = _measurer.default.getTree();

  var treeSnapshot = (0, _getTreeSnapshot.default)(roots, tree);
  var aggregatedStats = {};
  var affectedIDs = {};

  function updateAggregatedStats(treeSnapshot, instanceID, applyUpdate) {
    var component = treeSnapshot[instanceID];

    if (!component) {
      return;
    }

    var displayName = component.displayName,
        parentID = component.parentID;
    var owner = treeSnapshot[parentID];
    var key = (owner ? owner.displayName + ' > ' : '') + displayName;
    var stats = aggregatedStats[key];

    if (!stats) {
      affectedIDs[key] = {};
      stats = aggregatedStats[key] = {
        key: key,
        instanceCount: 0,
        inclusiveRenderDuration: 0,
        renderCount: 0
      };
    }

    affectedIDs[key][instanceID] = true;
    applyUpdate(stats);
  }

  var isCompositeByID = {};
  data.forEach(function (flush) {
    var measurements = flush.measurements;
    measurements.forEach(function (measurement) {
      var duration = measurement.duration,
          instanceID = measurement.instanceID,
          timerType = measurement.timerType;

      if (timerType !== 'render') {
        return;
      }

      isCompositeByID[instanceID] = true;
      var nextParentID = instanceID;

      while (nextParentID) {
        if (isCompositeByID[nextParentID]) {
          updateAggregatedStats(treeSnapshot, nextParentID, function (stats) {
            stats.inclusiveRenderDuration += duration;
          });
        }

        if (treeSnapshot[nextParentID]) {
          nextParentID = treeSnapshot[nextParentID].parentID;
        } else {
          nextParentID = 0;
        }
      }
    });
  });
  return Object.keys(aggregatedStats).map(function (key) {
    return _objectSpread({}, aggregatedStats[key], {
      instanceCount: Object.keys(affectedIDs[key]).length
    });
  }).sort(function (a, b) {
    return b.inclusiveRenderDuration - a.inclusiveRenderDuration;
  });
}

function getWasted() {
  var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getLastMeasurements();

  if (process.env.NODE_ENV === 'production') {
    return [];
  }

  var roots = _measurer.default.getRoots();

  var tree = _measurer.default.getTree();

  var treeSnapshot = (0, _getTreeSnapshot.default)(roots, tree);
  var aggregatedStats = {};
  var affectedIDs = {};

  function updateAggregatedStats(treeSnapshot, instanceID, applyUpdate) {
    var component = treeSnapshot[instanceID];

    if (!component) {
      return;
    }

    var displayName = component.displayName,
        parentID = component.parentID;
    var owner = treeSnapshot[parentID];
    var key = (owner ? owner.displayName + ' > ' : '') + displayName;
    var stats = aggregatedStats[key];

    if (!stats) {
      affectedIDs[key] = {};
      stats = aggregatedStats[key] = {
        key: key,
        instanceCount: 0,
        inclusiveRenderDuration: 0,
        renderCount: 0
      };
    }

    affectedIDs[key][instanceID] = true;
    applyUpdate(stats);
  }

  data.forEach(function (flush) {
    var measurements = flush.measurements,
        operations = flush.operations;
    var isDefinitelyNotWastedByID = {};
    operations.forEach(function (operation) {
      var instanceID = operation.instanceID;
      var nextParentID = instanceID;

      while (nextParentID) {
        isDefinitelyNotWastedByID[nextParentID] = true;
        nextParentID = treeSnapshot[nextParentID] ? treeSnapshot[nextParentID].parentID : null;
      }
    });
    var renderedCompositeIDs = {};
    measurements.forEach(function (measurement) {
      var instanceID = measurement.instanceID,
          timerType = measurement.timerType;

      if (timerType !== 'render') {
        return;
      }

      renderedCompositeIDs[instanceID] = true;
    });
    measurements.forEach(function (measurement) {
      var duration = measurement.duration,
          instanceID = measurement.instanceID,
          timerType = measurement.timerType;
      var component = treeSnapshot[instanceID];

      if (!component) {
        return;
      }

      if (timerType !== 'render') {
        return;
      }

      var updateCount = component.updateCount;

      if (isDefinitelyNotWastedByID[instanceID] || updateCount === 0) {
        return;
      }

      updateAggregatedStats(treeSnapshot, instanceID, function (stats) {
        stats.renderCount++;
      });
      var nextParentID = instanceID;

      while (nextParentID) {
        var isWasted = renderedCompositeIDs[nextParentID] && !isDefinitelyNotWastedByID[nextParentID];

        if (isWasted) {
          updateAggregatedStats(treeSnapshot, nextParentID, function (stats) {
            stats.inclusiveRenderDuration += duration;
          });
        }

        nextParentID = treeSnapshot[nextParentID].parentID;
      }
    });
  });
  return Object.keys(aggregatedStats).map(function (key) {
    return _objectSpread({}, aggregatedStats[key], {
      instanceCount: Object.keys(affectedIDs[key]).length
    });
  }).sort(function (a, b) {
    return b.inclusiveRenderDuration - a.inclusiveRenderDuration;
  });
}

function getExclusive() {
  var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getLastMeasurements();

  if (process.env.NODE_ENV === 'production') {
    return [];
  }

  var aggregatedStats = {};
  var affectedIDs = {};

  var roots = _measurer.default.getRoots();

  var tree = _measurer.default.getTree();

  var treeSnapshot = (0, _getTreeSnapshot.default)(roots, tree);

  function updateAggregatedStats(treeSnapshot, instanceID, timerType, applyUpdate) {
    var component = treeSnapshot[instanceID];

    if (!component) {
      return;
    }

    var displayName = component.displayName;
    var key = displayName;
    var stats = aggregatedStats[key];

    if (!stats) {
      affectedIDs[key] = {};
      stats = aggregatedStats[key] = {
        key: key,
        instanceCount: 0,
        counts: {},
        durations: {},
        totalDuration: 0
      };
    }

    if (!stats.durations[timerType]) {
      stats.durations[timerType] = 0;
    }

    if (!stats.counts[timerType]) {
      stats.counts[timerType] = 0;
    }

    affectedIDs[key][instanceID] = true;
    applyUpdate(stats);
  }

  data.forEach(function (flush) {
    var measurements = flush.measurements;
    measurements.forEach(function (measurement) {
      var duration = measurement.duration,
          instanceID = measurement.instanceID,
          timerType = measurement.timerType;
      updateAggregatedStats(treeSnapshot, instanceID, timerType, function (stats) {
        stats.totalDuration += duration;
        stats.durations[timerType] += duration;
        stats.counts[timerType]++;
      });
    });
  });
  return Object.keys(aggregatedStats).map(function (key) {
    return _objectSpread({}, aggregatedStats[key], {
      instanceCount: Object.keys(affectedIDs[key]).length
    });
  }).sort(function (a, b) {
    return b.totalDuration - a.totalDuration;
  });
}

function printInclusive() {
  if (process.env.NODE_ENV === 'production') {
    return;
  }

  var stats = getInclusive();
  var table = stats.map(function (item) {
    var key = item.key,
        instanceCount = item.instanceCount,
        inclusiveRenderDuration = item.inclusiveRenderDuration,
        renderCount = item.renderCount;
    return {
      'Component': key,
      'Inclusive time (ms)': roundFloat(inclusiveRenderDuration),
      'Instance count': instanceCount
    };
  });
  console.table(table);
}

function printExclusive() {
  if (process.env.NODE_ENV === 'production') {
    return;
  }

  var stats = getExclusive();
  var table = stats.map(function (item) {
    var key = item.key,
        instanceCount = item.instanceCount,
        totalDuration = item.totalDuration;
    var renderCount = item.counts.render || 0;
    var renderDuration = item.durations.render || 0;
    return {
      'Component': key,
      'Total time (ms)': roundFloat(totalDuration),
      'Instance count': instanceCount,
      'Total render time (ms)': roundFloat(renderDuration),
      'Average render time (ms)': renderCount ? roundFloat(renderDuration / renderCount) : undefined,
      'Render count': renderCount,
      'Total lifecycle time (ms)': roundFloat(totalDuration - renderDuration)
    };
  });
  console.table(table);
}

function printOperations() {
  if (process.env.NODE_ENV === 'production') {
    return;
  }

  var stats = getOperations();
  var table = stats.map(function (stat) {
    return {
      'Owner > Node': stat.key,
      'Operation': stat.type,
      'Payload': _typeof(stat.payload) === 'object' ? JSON.stringify(stat.payload) : stat.payload,
      'Flush index': stat.flushIndex,
      'Owner Component ID': stat.parentID,
      'DOM Component ID': stat.instanceID
    };
  });
  console.table(table);
}

function printWasted() {
  if (process.env.NODE_ENV === 'production') {
    return;
  }

  var stats = getWasted();
  var table = stats.map(function (item) {
    var key = item.key,
        instanceCount = item.instanceCount,
        inclusiveRenderDuration = item.inclusiveRenderDuration,
        renderCount = item.renderCount;
    return {
      'Owner > Component': key,
      'wasted time (ms)': roundFloat(inclusiveRenderDuration),
      'Instance count': instanceCount,
      'Render count': renderCount
    };
  });
  console.table(table);
}

var _default = {
  getLastMeasurements: getLastMeasurements,
  getExclusive: getExclusive,
  getInclusive: getInclusive,
  getWasted: getWasted,
  getOperations: getOperations,
  printExclusive: printExclusive,
  printInclusive: printInclusive,
  printWasted: printWasted,
  printOperations: printOperations,
  start: start,
  stop: stop,
  Measurer: _measurer.default
};
exports.default = _default;
module.exports = exports["default"];