"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var profiling = false;
var currentNesting = 0;
var currentStartTime = 0;
var currentNestedDuration = 0;
var currentMeasurements = [];
var operations = [];
var data = [];
var tree = {};
var roots = {};
var performance = {};

if (typeof window !== 'undefined') {
  performance = window.performance || window.webkitPerformance || {};
}

function now() {
  if (performance.now) {
    return performance.now();
  } else {
    return Date.now();
  }
}

function resetMeasurements() {
  if (currentMeasurements.length) {
    data.push({
      duration: now() - currentStartTime,
      measurements: currentMeasurements,
      operations: operations,
      tree: tree
    });
  }

  currentStartTime = now();
  currentMeasurements = [];
}

function clear() {
  currentStartTime = 0;
  currentNestedDuration = 0;
}

var Measurer = {
  begin: function begin() {
    if (profiling) {
      return;
    }

    profiling = true;
    data = [];
    resetMeasurements();
  },
  end: function end() {
    if (!profiling) {
      return;
    }

    profiling = false;
    resetMeasurements();
  },
  beforeRender: function beforeRender() {
    currentNesting++;
    resetMeasurements();
    clear();
  },
  afterRender: function afterRender() {
    resetMeasurements();
    currentNesting--;
  },
  beforeMountComponent: function beforeMountComponent(instanceID, element) {
    tree[instanceID] = element; // root

    if (element.getName() === 'Root') {
      roots[instanceID] = element;
    }
  },
  afterMountComponent: function afterMountComponent(instanceID) {
    var item = tree[instanceID];
    item.isMounted = true;
  },
  beforeUpdateComponent: function beforeUpdateComponent(instanceID, element) {
    var item = tree[instanceID];

    if (!item || !item.isMounted) {
      return;
    }

    item.element = element;
  },
  afterUpdateComponent: function afterUpdateComponent(instanceID) {
    var item = tree[instanceID];

    if (!item || !item.isMounted) {
      return;
    }

    item.updateCount++;
  },
  beforeLifeCycle: function beforeLifeCycle(instanceID, timeType) {
    currentStartTime = now();
    currentNestedDuration = 0;
  },
  afterLifeCycle: function afterLifeCycle(instanceID, timerType) {
    if (profiling) {
      currentMeasurements.push({
        timerType: timerType,
        instanceID: instanceID,
        duration: now() - currentStartTime - currentNestedDuration
      });
    }

    clear();
  },
  recordOperation: function recordOperation(operation) {
    operations.push(operation);
  },
  getData: function getData() {
    return data;
  },
  getRoots: function getRoots() {
    return roots;
  },
  getTree: function getTree() {
    return tree;
  }
};
var _default = Measurer;
exports.default = _default;
module.exports = exports["default"];